@@ -13,8 +13,8 @@
 #    under the License.
 
 import mock
+from os_brick import encryptors
 from oslo_serialization import jsonutils
-import six
 
 from nova import block_device
 from nova import context
@@ -29,10 +29,13 @@ from nova.tests import uuidsentinel as uuids
 from nova.virt import block_device as driver_block_device
 from nova.virt import driver
 from nova.volume import cinder
-from nova.volume import encryptors
 
 
 class TestDriverBlockDevice(test.NoDBTestCase):
+    # This is used to signal if we're dealing with a new style volume
+    # attachment (Cinder v3.44 flow).
+    attachment_id = None
+
     driver_classes = {
         'swap': driver_block_device.DriverSwapBlockDevice,
         'ephemeral': driver_block_device.DriverEphemeralBlockDevice,
@@ -219,6 +222,14 @@ class TestDriverBlockDevice(test.NoDBTestCase):
         self.blank_bdm = fake_block_device.fake_bdm_object(
             self.context, self.blank_bdm_dict)
 
+        # Set the attachment_id on our fake class variables which we have
+        # to do in setUp so that any attachment_id set by a subclass will
+        # be used properly.
+        for name in ('volume', 'snapshot', 'image', 'blank'):
+            for attr in ('%s_bdm', '%s_driver_bdm'):
+                bdm = getattr(self, attr % name)
+                bdm['attachment_id'] = self.attachment_id
+
     def test_no_device_raises(self):
         for name, cls in self.driver_classes.items():
             bdm = fake_block_device.fake_bdm_object(
@@ -232,7 +243,7 @@ class TestDriverBlockDevice(test.NoDBTestCase):
         self.assertThat(test_bdm, matchers.DictMatches(
             getattr(self, "%s_driver_bdm" % name)))
 
-        for k, v in six.iteritems(db_bdm):
+        for k, v in db_bdm.items():
             field_val = getattr(test_bdm._bdm_obj, k)
             if isinstance(field_val, bool):
                 v = bool(v)
@@ -248,7 +259,7 @@ class TestDriverBlockDevice(test.NoDBTestCase):
                              getattr(test_bdm._bdm_obj, passthru))
 
         # Make sure that all others raise _invalidType
-        for other_name, cls in six.iteritems(self.driver_classes):
+        for other_name, cls in self.driver_classes.items():
             if other_name == name:
                 continue
             self.assertRaises(driver_block_device._InvalidType,
@@ -257,14 +268,14 @@ class TestDriverBlockDevice(test.NoDBTestCase):
 
         # Test the save method
         with mock.patch.object(test_bdm._bdm_obj, 'save') as save_mock:
-            for fld, alias in six.iteritems(test_bdm._update_on_save):
+            for fld, alias in test_bdm._update_on_save.items():
                 # We can't set fake values on enums, like device_type,
                 # so skip those.
                 if not isinstance(test_bdm._bdm_obj.fields[fld],
                                   fields.BaseEnumField):
                     test_bdm[alias or fld] = 'fake_changed_value'
             test_bdm.save()
-            for fld, alias in six.iteritems(test_bdm._update_on_save):
+            for fld, alias in test_bdm._update_on_save.items():
                 self.assertEqual(test_bdm[alias or fld],
                                  getattr(test_bdm._bdm_obj, fld))
 
@@ -384,12 +395,57 @@ class TestDriverBlockDevice(test.NoDBTestCase):
     def test_call_wait_no_delete_volume(self):
         self._test_call_wait_func(False)
 
+    def test_volume_delete_attachment(self, include_shared_targets=False):
+        attachment_id = uuids.attachment
+        driver_bdm = self.driver_classes['volume'](self.volume_bdm)
+        driver_bdm['attachment_id'] = attachment_id
+
+        elevated_context = self.context.elevated()
+        instance_detail = {'id': '123', 'uuid': uuids.uuid,
+                           'availability_zone': None}
+        instance = fake_instance.fake_instance_obj(self.context,
+                                                   **instance_detail)
+        connector = {'ip': 'fake_ip', 'host': 'fake_host'}
+        volume = {'id': driver_bdm.volume_id,
+                  'attach_status': 'attached',
+                  'status': 'in-use'}
+        if include_shared_targets:
+            volume['shared_targets'] = True
+            volume['service_uuid'] = uuids.service_uuid
+
+        with test.nested(
+            mock.patch.object(driver_bdm, '_get_volume', return_value=volume),
+            mock.patch.object(self.virt_driver, 'get_volume_connector',
+                              return_value=connector),
+            mock.patch('nova.utils.synchronized',
+                side_effect=lambda a: lambda f: lambda *args: f(*args)),
+            mock.patch.object(self.volume_api, 'attachment_delete'),
+        ) as (mock_get_volume, mock_get_connector, mock_sync, vapi_attach_del):
+
+            driver_bdm.detach(elevated_context, instance,
+                              self.volume_api, self.virt_driver,
+                              attachment_id=attachment_id)
+
+            if include_shared_targets:
+                mock_sync.assert_called_once_with((uuids.service_uuid))
+            vapi_attach_del.assert_called_once_with(elevated_context,
+                                                    attachment_id)
+
+    def test_volume_delete_attachment_with_shared_targets(self):
+        self.test_volume_delete_attachment(include_shared_targets=True)
+
     def _test_volume_attach(self, driver_bdm, bdm_dict,
-                            fake_volume, check_attach=True,
-                            fail_check_attach=False, driver_attach=False,
-                            fail_driver_attach=False, volume_attach=True,
-                            fail_volume_attach=False, access_mode='rw',
-                            availability_zone=None):
+                            fake_volume, fail_check_av_zone=False,
+                            driver_attach=False, fail_driver_attach=False,
+                            volume_attach=True, fail_volume_attach=False,
+                            access_mode='rw', availability_zone=None,
+                            multiattach=False, driver_multi_attach=False,
+                            fail_with_virt_driver=False,
+                            include_shared_targets=False):
+        if driver_multi_attach:
+            self.virt_driver.capabilities['supports_multiattach'] = True
+        else:
+            self.virt_driver.capabilities['supports_multiattach'] = False
         elevated_context = self.context.elevated()
         self.stubs.Set(self.context, 'elevated',
                        lambda: elevated_context)
@@ -403,25 +459,54 @@ class TestDriverBlockDevice(test.NoDBTestCase):
         connection_info = {'data': {'access_mode': access_mode}}
         expected_conn_info = {'data': {'access_mode': access_mode},
                               'serial': fake_volume['id']}
+        if multiattach and driver_multi_attach:
+            expected_conn_info['multiattach'] = True
         enc_data = {'fake': 'enc_data'}
 
-        self.volume_api.get(self.context,
-                            fake_volume['id']).AndReturn(fake_volume)
-        if check_attach:
-            if not fail_check_attach:
-                self.volume_api.check_attach(self.context, fake_volume,
-                                    instance=instance).AndReturn(None)
-            else:
-                self.volume_api.check_attach(self.context, fake_volume,
-                                    instance=instance).AndRaise(
-                                            test.TestingException)
-                driver_bdm._bdm_obj.save().AndReturn(None)
-                return instance, expected_conn_info
+        if include_shared_targets:
+            fake_volume['shared_targets'] = True
+            fake_volume['service_uuid'] = uuids.service_uuid
+            self.volume_api.get(
+                self.context, fake_volume['id'],
+                microversion='3.48').AndReturn(fake_volume)
+        else:
+            # First call to get() fails because the API isn't new enough.
+            self.volume_api.get(
+                self.context, fake_volume['id'], microversion='3.48').AndRaise(
+                    exception.CinderAPIVersionNotAvailable(version='3.48'))
+            # So we fallback to the old call.
+            self.volume_api.get(self.context,
+                                fake_volume['id']).AndReturn(fake_volume)
+
+        if not fail_check_av_zone:
+            self.volume_api.check_availability_zone(self.context,
+                                fake_volume,
+                                instance=instance).AndReturn(None)
+        else:
+            self.volume_api.check_availability_zone(self.context,
+                                fake_volume,
+                                instance=instance).AndRaise(
+                                        test.TestingException)
+            # The @update_db decorator will save any changes.
+            driver_bdm._bdm_obj.save().AndReturn(None)
+            return instance, expected_conn_info
 
         self.virt_driver.get_volume_connector(instance).AndReturn(connector)
-        self.volume_api.initialize_connection(
-            elevated_context, fake_volume['id'],
-            connector).AndReturn(connection_info)
+
+        if fail_with_virt_driver:
+            driver_bdm._bdm_obj.save().AndReturn(None)
+            return instance, expected_conn_info
+
+        if self.attachment_id is None:
+            self.volume_api.initialize_connection(
+                elevated_context, fake_volume['id'],
+                connector).AndReturn(connection_info)
+        else:
+            self.volume_api.attachment_update(
+                elevated_context, self.attachment_id, connector,
+                bdm_dict['device_name']).AndReturn(
+                    {'connection_info': connection_info})
+
         if driver_attach:
             encryptors.get_encryption_metadata(
                     elevated_context, self.volume_api, fake_volume['id'],
@@ -440,45 +525,64 @@ class TestDriverBlockDevice(test.NoDBTestCase):
                         disk_bus=bdm_dict['disk_bus'],
                         device_type=bdm_dict['device_type'],
                         encryption=enc_data).AndRaise(test.TestingException)
-                self.volume_api.terminate_connection(
-                        elevated_context, fake_volume['id'],
-                        connector).AndReturn(None)
+                if self.attachment_id is None:
+                    self.volume_api.terminate_connection(
+                            elevated_context, fake_volume['id'],
+                            connector).AndReturn(None)
+                else:
+                    self.volume_api.attachment_delete(
+                        elevated_context, self.attachment_id).AndReturn(None)
+                # The @update_db decorator will save any changes.
                 driver_bdm._bdm_obj.save().AndReturn(None)
                 return instance, expected_conn_info
 
         if volume_attach:
+            # save updates before marking the volume as in-use
             driver_bdm._bdm_obj.save().AndReturn(None)
             if not fail_volume_attach:
-                self.volume_api.attach(elevated_context, fake_volume['id'],
-                                       uuids.uuid, bdm_dict['device_name'],
-                                        mode=access_mode).AndReturn(None)
+                if self.attachment_id is None:
+                    self.volume_api.attach(elevated_context, fake_volume['id'],
+                                           uuids.uuid, bdm_dict['device_name'],
+                                            mode=access_mode).AndReturn(None)
+                else:
+                    self.volume_api.attachment_complete(
+                        elevated_context, self.attachment_id).AndReturn(None)
             else:
-                self.volume_api.attach(elevated_context, fake_volume['id'],
-                                       uuids.uuid, bdm_dict['device_name'],
-                                        mode=access_mode).AndRaise(
-                                            test.TestingException)
-                if driver_attach:
-                    self.virt_driver.detach_volume(
+                if self.attachment_id is None:
+                    self.volume_api.attach(elevated_context, fake_volume['id'],
+                                           uuids.uuid, bdm_dict['device_name'],
+                                            mode=access_mode).AndRaise(
+                                                test.TestingException)
+                    if driver_attach:
+                        self.virt_driver.detach_volume(
                             expected_conn_info, instance,
                             bdm_dict['device_name'],
                             encryption=enc_data).AndReturn(None)
-                self.volume_api.terminate_connection(
+                    self.volume_api.terminate_connection(
                         elevated_context, fake_volume['id'],
                         connector).AndReturn(None)
-                self.volume_api.detach(elevated_context,
-                                       fake_volume['id']).AndReturn(None)
+                    self.volume_api.detach(elevated_context,
+                                           fake_volume['id']).AndReturn(None)
+                else:
+                    self.volume_api.attachment_complete(
+                        elevated_context, self.attachment_id).AndRaise(
+                            test.TestingException)
+                    self.volume_api.attachment_delete(
+                        elevated_context, self.attachment_id).AndReturn(None)
 
+        # The @update_db decorator will save any changes.
         driver_bdm._bdm_obj.save().AndReturn(None)
         return instance, expected_conn_info
 
-    def test_volume_attach(self):
+    def test_volume_attach(self, include_shared_targets=False):
         test_bdm = self.driver_classes['volume'](
             self.volume_bdm)
         volume = {'id': 'fake-volume-id-1',
                   'attach_status': 'detached'}
 
         instance, expected_conn_info = self._test_volume_attach(
-                test_bdm, self.volume_bdm, volume)
+                test_bdm, self.volume_bdm, volume,
+                include_shared_targets=include_shared_targets)
 
         self.mox.ReplayAll()
 
@@ -487,6 +591,9 @@ class TestDriverBlockDevice(test.NoDBTestCase):
         self.assertThat(test_bdm['connection_info'],
                         matchers.DictMatches(expected_conn_info))
 
+    def test_volume_attach_with_shared_targets(self):
+        self.test_volume_attach(include_shared_targets=True)
+
     def test_volume_attach_ro(self):
         test_bdm = self.driver_classes['volume'](self.volume_bdm)
         volume = {'id': 'fake-volume-id-1',
@@ -519,13 +626,13 @@ class TestDriverBlockDevice(test.NoDBTestCase):
         self.assertEqual(expected_conn_info, test_bdm['connection_info'])
         self.assertEqual(42, test_bdm.volume_size)
 
-    def test_volume_attach_check_attach_fails(self):
+    def test_volume_attach_check_av_zone_fails(self):
         test_bdm = self.driver_classes['volume'](
             self.volume_bdm)
         volume = {'id': 'fake-volume-id-1'}
 
         instance, _ = self._test_volume_attach(
-                test_bdm, self.volume_bdm, volume, fail_check_attach=True)
+                test_bdm, self.volume_bdm, volume, fail_check_av_zone=True)
         self.mox.ReplayAll()
 
         self.assertRaises(test.TestingException, test_bdm.attach, self.context,
@@ -538,14 +645,13 @@ class TestDriverBlockDevice(test.NoDBTestCase):
                   'attach_status': 'detached'}
 
         instance, expected_conn_info = self._test_volume_attach(
-                test_bdm, self.volume_bdm, volume, check_attach=False,
-                driver_attach=False)
+                test_bdm, self.volume_bdm, volume, driver_attach=False)
 
         self.mox.ReplayAll()
 
         test_bdm.attach(self.context, instance,
                         self.volume_api, self.virt_driver,
-                        do_check_attach=False, do_driver_attach=False)
+                        do_driver_attach=False)
         self.assertThat(test_bdm['connection_info'],
                         matchers.DictMatches(expected_conn_info))
 
@@ -556,14 +662,13 @@ class TestDriverBlockDevice(test.NoDBTestCase):
                   'attach_status': 'detached'}
 
         instance, expected_conn_info = self._test_volume_attach(
-                test_bdm, self.volume_bdm, volume, check_attach=False,
-                driver_attach=True)
+                test_bdm, self.volume_bdm, volume, driver_attach=True)
 
         self.mox.ReplayAll()
 
         test_bdm.attach(self.context, instance,
                         self.volume_api, self.virt_driver,
-                        do_check_attach=False, do_driver_attach=True)
+                        do_driver_attach=True)
         self.assertThat(test_bdm['connection_info'],
                         matchers.DictMatches(expected_conn_info))
 
@@ -622,10 +727,16 @@ class TestDriverBlockDevice(test.NoDBTestCase):
 
         self.mox.StubOutWithMock(test_bdm._bdm_obj, 'save')
 
-        self.virt_driver.get_volume_connector(instance).AndReturn(connector)
-        self.volume_api.initialize_connection(
-            self.context, test_bdm.volume_id,
-            connector).AndReturn(connection_info)
+        if self.attachment_id is None:
+            self.virt_driver.get_volume_connector(instance).AndReturn(
+                connector)
+            self.volume_api.initialize_connection(
+                self.context, test_bdm.volume_id,
+                connector).AndReturn(connection_info)
+        else:
+            self.volume_api.attachment_get(
+                self.context, self.attachment_id).AndReturn(
+                    {'connection_info': connection_info})
         test_bdm._bdm_obj.save().AndReturn(None)
 
         self.mox.ReplayAll()
@@ -641,6 +752,8 @@ class TestDriverBlockDevice(test.NoDBTestCase):
         test_bdm = self.driver_classes['snapshot'](
                 fake_block_device.fake_bdm_object(
                         self.context, no_volume_snapshot))
+        # When we create a volume, we attach it using the old flow.
+        self.attachment_id = None
 
         snapshot = {'id': 'fake-volume-id-1',
                     'attach_status': 'detached'}
@@ -671,6 +784,8 @@ class TestDriverBlockDevice(test.NoDBTestCase):
         test_bdm = self.driver_classes['snapshot'](
                 fake_block_device.fake_bdm_object(
                         self.context, no_volume_snapshot))
+        # When we create a volume, we attach it using the old flow.
+        self.attachment_id = None
 
         snapshot = {'id': 'fake-volume-id-1',
                     'attach_status': 'detached'}
@@ -746,8 +861,7 @@ class TestDriverBlockDevice(test.NoDBTestCase):
         self.mox.StubOutWithMock(self.volume_api, 'create')
 
         volume_class.attach(self.context, instance, self.volume_api,
-                            self.virt_driver, do_check_attach=True
-                            ).AndReturn(None)
+                            self.virt_driver).AndReturn(None)
         self.mox.ReplayAll()
 
         test_bdm.attach(self.context, instance, self.volume_api,
@@ -760,6 +874,8 @@ class TestDriverBlockDevice(test.NoDBTestCase):
         test_bdm = self.driver_classes['image'](
                 fake_block_device.fake_bdm_object(
                         self.context, no_volume_image))
+        # When we create a volume, we attach it using the old flow.
+        self.attachment_id = None
 
         image = {'id': 'fake-image-id-1'}
         volume = {'id': 'fake-volume-id-2',
@@ -787,6 +903,8 @@ class TestDriverBlockDevice(test.NoDBTestCase):
         test_bdm = self.driver_classes['image'](
                 fake_block_device.fake_bdm_object(
                         self.context, no_volume_image))
+        # When we create a volume, we attach it using the old flow.
+        self.attachment_id = None
 
         image = {'id': 'fake-image-id-1'}
         volume = {'id': 'fake-volume-id-2',
@@ -855,8 +973,7 @@ class TestDriverBlockDevice(test.NoDBTestCase):
         self.mox.StubOutWithMock(self.volume_api, 'create')
 
         volume_class.attach(self.context, instance, self.volume_api,
-                            self.virt_driver, do_check_attach=True
-                            ).AndReturn(None)
+                            self.virt_driver).AndReturn(None)
         self.mox.ReplayAll()
 
         test_bdm.attach(self.context, instance, self.volume_api,
@@ -923,8 +1040,7 @@ class TestDriverBlockDevice(test.NoDBTestCase):
                 '', availability_zone=None)
             vol_attach.assert_called_once_with(self.context, instance,
                                                self.volume_api,
-                                               self.virt_driver,
-                                               do_check_attach=True)
+                                               self.virt_driver)
             self.assertEqual('fake-volume-id-2', test_bdm.volume_id)
 
     def test_blank_attach_volume_cinder_cross_az_attach_false(self):
@@ -955,8 +1071,7 @@ class TestDriverBlockDevice(test.NoDBTestCase):
                     '', availability_zone='test-az')
                 vol_attach.assert_called_once_with(self.context, instance,
                                                    self.volume_api,
-                                                   self.virt_driver,
-                                                   do_check_attach=True)
+                                                   self.virt_driver)
                 self.assertEqual('fake-volume-id-2', test_bdm.volume_id)
 
     def test_convert_block_devices(self):
@@ -1052,3 +1167,135 @@ class TestDriverBlockDevice(test.NoDBTestCase):
         instance = fake_instance.fake_instance_obj(self.context, **updates)
         self.assertIsNone(
             driver_block_device._get_volume_create_az_value(instance))
+
+    def test_refresh_conn_infos(self):
+        # Only DriverVolumeBlockDevice derived devices should refresh their
+        # connection_info during a refresh_conn_infos call.
+        test_volume = mock.MagicMock(
+            spec=driver_block_device.DriverVolumeBlockDevice)
+        test_image = mock.MagicMock(
+            spec=driver_block_device.DriverImageBlockDevice)
+        test_snapshot = mock.MagicMock(
+            spec=driver_block_device.DriverSnapshotBlockDevice)
+        test_blank = mock.MagicMock(
+            spec=driver_block_device.DriverBlankBlockDevice)
+        test_eph = mock.MagicMock(
+            spec=driver_block_device.DriverEphemeralBlockDevice)
+        test_swap = mock.MagicMock(
+            spec=driver_block_device.DriverSwapBlockDevice)
+        block_device_mapping = [test_volume, test_image, test_eph,
+                                test_snapshot, test_swap, test_blank]
+        driver_block_device.refresh_conn_infos(block_device_mapping,
+                                               mock.sentinel.refresh_context,
+                                               mock.sentinel.refresh_instance,
+                                               mock.sentinel.refresh_vol_api,
+                                               mock.sentinel.refresh_virt_drv)
+        for test_mock in [test_volume, test_image, test_snapshot, test_blank]:
+            test_mock.refresh_connection_info.assert_called_once_with(
+                mock.sentinel.refresh_context,
+                mock.sentinel.refresh_instance,
+                mock.sentinel.refresh_vol_api,
+                mock.sentinel.refresh_virt_drv)
+        # NOTE(lyarwood): Can't think of a better way of testing this as we
+        # can't assert_not_called if the method isn't in the spec.
+        self.assertFalse(hasattr(test_eph, 'refresh_connection_info'))
+        self.assertFalse(hasattr(test_swap, 'refresh_connection_info'))
+
+    def test_proxy_as_attr(self):
+        class A(driver_block_device.DriverBlockDevice):
+            pass
+
+            def _transform(self):
+                pass
+
+        class B(A):
+            _proxy_as_attr_inherited = set('B')
+
+        class C(A):
+            _proxy_as_attr_inherited = set('C')
+
+        class D(B):
+            _proxy_as_attr_inherited = set('D')
+
+        class E(B, C):
+            _proxy_as_attr_inherited = set('E')
+
+        bdm = objects.BlockDeviceMapping(self.context, no_device=False)
+        self.assertEqual(set(['uuid']), A(bdm)._proxy_as_attr)
+        self.assertEqual(set(['uuid', 'B']), B(bdm)._proxy_as_attr)
+        self.assertEqual(set(['uuid', 'C']), C(bdm)._proxy_as_attr)
+        self.assertEqual(set(['uuid', 'B', 'D']), D(bdm)._proxy_as_attr)
+        self.assertEqual(set(['uuid', 'B', 'C', 'E']), E(bdm)._proxy_as_attr)
+
+
+class TestDriverBlockDeviceNewFlow(TestDriverBlockDevice):
+    """Virt block_device tests for the Cinder 3.44 volume attach flow
+    where a volume BDM has an attachment_id.
+    """
+    attachment_id = uuids.attachment_id
+
+    def test_volume_attach_multiattach(self):
+        test_bdm = self.driver_classes['volume'](
+            self.volume_bdm)
+        volume = {'id': 'fake-volume-id-1',
+                  'multiattach': True,
+                  'attach_status': 'attached',
+                  'status': 'in-use',
+                  'attachments': {'fake_instance_2':
+                                  {'mountpoint': '/dev/vdc'}}}
+
+        instance, expected_conn_info = self._test_volume_attach(
+                test_bdm, self.volume_bdm, volume, multiattach=True,
+                driver_multi_attach=True)
+
+        self.mox.ReplayAll()
+
+        test_bdm.attach(self.context, instance,
+                        self.volume_api, self.virt_driver)
+        self.assertThat(test_bdm['connection_info'],
+                        matchers.DictMatches(expected_conn_info))
+
+    def test_volume_attach_multiattach_no_virt_driver_support(self):
+        test_bdm = self.driver_classes['volume'](
+            self.volume_bdm)
+        volume = {'id': 'fake-volume-id-1',
+                  'multiattach': True,
+                  'attach_status': 'attached',
+                  'status': 'in-use',
+                  'attachments': {'fake_instance_2':
+                                  {'mountpoint': '/dev/vdc'}}}
+
+        instance, _ = self._test_volume_attach(test_bdm, self.volume_bdm,
+                                               volume, multiattach=True,
+                                               fail_with_virt_driver=True)
+
+        self.mox.ReplayAll()
+
+        self.assertRaises(exception.MultiattachNotSupportedByVirtDriver,
+                          test_bdm.attach, self.context, instance,
+                          self.volume_api, self.virt_driver)
+
+
+class TestGetVolumeId(test.NoDBTestCase):
+
+    def test_get_volume_id_none_found(self):
+        self.assertIsNone(driver_block_device.get_volume_id(None))
+        self.assertIsNone(driver_block_device.get_volume_id({}))
+        self.assertIsNone(driver_block_device.get_volume_id({'data': {}}))
+
+    def test_get_volume_id_found_volume_id_no_serial(self):
+        self.assertEqual(uuids.volume_id,
+                         driver_block_device.get_volume_id(
+                             {'data': {'volume_id': uuids.volume_id}}))
+
+    def test_get_volume_id_found_no_volume_id_serial(self):
+        self.assertEqual(uuids.serial,
+                         driver_block_device.get_volume_id(
+                             {'serial': uuids.serial}))
+
+    def test_get_volume_id_found_both(self):
+        # volume_id is taken over serial
+        self.assertEqual(uuids.volume_id,
+                         driver_block_device.get_volume_id(
+                             {'serial': uuids.serial,
+                              'data': {'volume_id': uuids.volume_id}}))
